#include "ndominated.h"
#include "VectorSet.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <ctime>
#include <cstdlib>

using namespace std;

void readDataSet(char *fileName, int dim, VectorSet &solSet);
void writeDataSet(char *fileName, VectorSet solSet);
void printArray(vector<double> array[], int size);

int main(int argc, char **argv) {
	       /**< Input vector set. */
	VectorSet nDomKung;      /**< Nondominated solutions. */
	int dimension;
	
	if (argc < 4) {
		cout << "\nUsage: \n" << "$> " << argv[0] 
		     << " <data set file> <no. objectives> <output file>\n";
		exit(0);
	}
	
	// Read file of solutions.
	istringstream dimStr(argv[2]);
	dimStr >> dimension;
	
	VectorSet dataSet;	
	readDataSet(argv[1], dimension, dataSet);
				
	vector<double> *arrayDataSet;
	arrayDataSet = new vector<double>[dataSet.size()];	
	dataSet.toArray(arrayDataSet);
	
	NonDominance checkDom;  

	clock_t start = clock();		
	checkDom.kungNDominatedSet(arrayDataSet, dataSet.size(), nDomKung);
	clock_t end = clock();
	
	cout << "Execution time: " 
	     << setw(15) << setprecision(10) << std::fixed
	     << (double) (end-start) / CLOCKS_PER_SEC << " sec" << endl;
	
	cout << "\nThere are " << nDomKung.size() << " nondominated vectors." << endl;

	writeDataSet(argv[3], nDomKung);
		
	delete [] arrayDataSet;
	
	return 0;
}

void printArray(vector<double> array[], int size) {
	for (int i = 0; i < size; ++i) {
		for (unsigned j = 0; j < array[i].size(); ++j) {
			cout << array[i].at(j) << " ";
		}
		cout << endl;
	}
}

/**
 * Function that reads a file of points into an object to store a set of vectors.
 */
void readDataSet(char *fileName, int dim, VectorSet &solSet) {
	ifstream dataFile(fileName, ios::in);
	string line;
	istringstream lineStream;
	vector<double> v(dim, 0);
	
	while ( getline(dataFile, line) ) {		
		lineStream.clear();
		lineStream.str(line);
		
		if (line.size() == 0 )
			break;
		
		//Leer cada componente de la solución y guardarla en un vector
		for (int i = 0; i < dim; ++i) 
			lineStream >> v[i];

	
		//Agregar el vector solución actual a solSet
		solSet.addElement(v);
	}
	
	dataFile.close();
}


/**
 * Function that write into a file the vectors in the VectorSet object.
 */
void writeDataSet(char *fileName, VectorSet vSet) {
	ofstream dataFile(fileName, ios::out); 

	dataFile << vSet.toString();
	
	dataFile.close();	
}

